;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;Error handlers

err_handlermem:
	ld hl,str_handlermem
	jp error

err_noprog:
	ld hl,str_noprog
	jp error

err_version:
	ld hl,str_version
	jp error

err_map8:
	ld hl,str_map8
	jp error

err_maplen:
	ld hl,str_maplen
	jp error

err_map:
	ld hl,str_map
	jp error

err_mem:
	ld hl,str_mem
	jp error

err_ptr:
	ld hl,str_ptr
	jp error

err_noptr:
	ld hl,str_noptr
	jp error

err_cint:
	ld hl,str_cint
	jp error

err_frun:
	ld hl,str_frun
	jp error

err_argrange:
	ld hl,str_argrange
	jp error

err_syntax:
	ld hl,str_syntax
	jp error

err_eol:
	ld hl,str_eol
	jp error

err_var:
	ld hl,str_var
	jp error

err_var_int:
	ld hl,str_var_int
	jp error

err_var_arry:
	ld hl,str_var_arry
	jp error

err_var_str:
	ld hl,str_var_str
	jp error

err_vartype:
	ld hl,str_vartype
	jp error

err_num:
	ld hl,str_num
	jp error

err_op:
	ld hl,str_op
	jp error

err_spr_prop:
	ld hl,str_spr_prop
	jp error

err_bmp:
	ld hl,str_bmp
	jp error

err_spr:
	ld hl,str_spr
	jp error

err_str:
	ld hl,str_str
	jp error

err_nomain:
	ld hl,str_nomain
	jp error

err_invcmd:
	ld hl,str_invcmd
	jp error

err_invarg:
	ld hl,str_invarg
	jp error

err_invop:
	ld hl,str_invop
	jp error

err_namelen:
	ld hl,str_namelen
	jp error

err_numlen:
	ld hl,str_numlen
	jp error

err_mem_var:
	ld hl,str_mem_var
	jp error

err_mem_lbl:
	ld hl,str_mem_lbl
	jr error

err_nolbl:
	ld hl,str_nolbl
	jr error

err_keycode:
	ld hl,str_keycode
	jr error

err_num_key:
	ld hl,str_num_key
	jr error

err_mem_file:
	ld hl,str_mem_file
	jr error

err_arraylen:
	ld hl,str_arraylen
	jr error

err_arrayindex:
	ld hl,str_arrayindex
	jr error

err_noarray:
	ld hl,str_noarray
	jr error

err_strlen:
	ld hl,str_strlen
	jr error

err_mem_bmp:
	ld hl,str_mem_bmp
	jr error

err_bmpformat:
	ld hl,str_sprformat
	jr error

err_nobmp:
	ld hl,str_nobmp
	jr error

err_mem_spr:
	ld hl,str_mem_spr
	jr error

err_nospr:
	ld hl,str_nospr
	jr error

err_twovar_arry:
	ld hl,str_twovar_arry
	jr error

err_twovar_bmp:
	ld hl,str_twovar_spr
	jr error

err_arraydeflen:
	ld hl,str_deflen
	jr error

err_int:
	ld hl,str_int
	jr error

err_progname:	;"name too long"
	ld hl,write_addr
	ld (write_ptr),hl
	ld hl,func_stack
	ld (func_sp),hl
	ld hl,str_progname
;	jr error
error:
	push hl
	ld a,$0d
	out (5),a
	call close_gray							;close grayscale handler
	call _clrlcd							;clear the screen
	ld hl,err_str_name-1
	ld b,0
	ld de,err_load_addr
	call load_file							;load the string with the error handler and text
	call _ram_page_1
	pop hl
	jp c,error_end
	push hl
	ld hl,err_load_addr
	ld de,err_copy_addr
	call Decompress							;and decompress it,
	ld de,error_str_pos						;set cursor position
	ld (_pencol),de
	ld hl,str_err							;draw "error!" message,
	call _vputs
	ld de,error_str_pos_2
	ld (_pencol),de
	ld hl,str_line
	call _vputs
	ld hl,(write_ptr)						;(select pointer to use to find line number)
	ld a,(hl)
	cp eof_token
	jr z,error_use_parse
error_use_write:
	ld de,(write_ptr)
	jr error_use_ptr
error_use_parse:
	ld de,(parse_ptr)
error_use_ptr:
	ld hl,write_addr						;then line number,
	ld bc,1
error_loop_1:
	call cphlde
	jr nc,error_end_loop_1
	jr z,error_end_loop_1
	push de
	ld de,end_copy_addr
	call cphlde
	pop de
	jr nc,error_end_loop_1_err
	push bc
	call goto_eol
	pop bc
	cp eof_token
	jr z,error_end_loop_1_err
	inc hl
	inc bc
	jr error_loop_1
error_end_loop_1_err:
	ld bc,1
error_end_loop_1:
	dec bc
	ld l,c
	ld h,b
	ld (error_line),hl
	call DisplayHL
	ld de,error_str_pos_3
	ld (_pencol),de
	pop hl
	ld b,(hl)
	inc hl									;then the specific error message
error_loop_2:
	call _vputs
	call _vnewline
	djnz error_loop_2
	call _vnewline
error_line	equ	$+1
	ld de,0
	ld a,d
	or e
	jp z,error_endshowline
	ld hl,(write_ptr)						;and the contents of the line, if possible
	ld a,(hl)
	cp eof_token
	jr z,error_showline_tok
	ld hl,(parse_ptr)
error_loop_3:
	dec hl
	ld a,(hl)
	cp $d6
	jr nz,error_loop_3
	inc hl
error_loop_4:
	ld a,(hl)
	cp $d6
	jr z,error_endshowline
	call _vputmap
	inc hl
	jr error_loop_4
error_showline_tok:
	push de
	ld hl,prog_name_save-1
	rst 20h
	rst 10h
	ex de,hl
	ld a,b
	call _ahl_plus_2
	call _abs_to_page_asic
	set 6,a
	out (5),a
	inc a
	out (6),a
	ld de,-$4000
	add hl,de
	pop de
	ld bc,0
error_loop_5:
	ld a,(hl)
	cp command_char
	jr z,error_showline_tok_isline
	cp string_char
	jr z,error_showline_tok_isline
	cp int_char
	jr z,error_showline_tok_isline
	cp array_char
	jr z,error_showline_tok_isline
	cp sprite_char
	jr z,error_showline_tok_isline
	cp bitmap_char
	jr z,error_showline_tok_isline
	cp pointer_char
	jr z,error_showline_tok_isline
	cp map_char
	jr z,error_showline_tok_isline
error_loop_6:
	ld a,(hl)
	cp $d6
	inc hl
	jr nz,error_loop_6
	jr error_loop_5
error_showline_tok_isline:
	inc bc
	ld a,b
	cp d
	jr nz,error_loop_6
	ld a,c
	cp e
	jr nz,error_loop_6
	ld de,err_line_copy_addr
error_loop_7:
	ld a,(hl)
	ld (de),a
	cp $d6
	jr z,end_error_loop_7
	inc hl
	inc de
	jr error_loop_7
end_error_loop_7:
	ld hl,err_line_copy_addr
	ld a,$0d
	out (5),a
	ld a,$41
	out (6),a
	jr error_loop_4
error_endshowline:
	call _getkey							;wait for a key to be pressed,
error_end:
	call clr_free_mem
	ld sp,(sp_save)							;then restore the stack pointer
	ret										;and return

.end