;version 2.7
;6/28/2005 build 1
;(18 files, 9000 lines of code!!!)
;11.18 KB for prog

;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;Main code

.plugin bin
#include "86.h"
#include "key_equ.asm"
#include "mlcerr.inc"

;COMPILE OPTIONS

#define FAST_LINES							;optimized line drawing (78 bytes)
#define FAST_KEY							;pre-processed keypress detection (56 bytes)

;PROGRAM SETTINGS

cmd_len			equ		4					;number of letters in each command
num_max_len		equ		5					;max length of number
str_max_len		equ		35					;max length of string
name_max_len	equ		5					;max length of any name (function, var, sprite, etc.)
num_colors		equ		6					;number of colors
max_vars		equ		110					;max number of vars
max_lbls		equ		125					;max number of labels
max_sprites		equ		15
arg_separator	equ		Lcomma				;char used to separate arguments of commands
index_start_char	equ	LlParen				;char used for defining index of an array
index_stop_char	equ		LrParen
command_char	equ		Lpound				;char that precedes commands
string_char		equ		Ldollar				;chars that precede different var/literal types
int_char		equ		Lpercent
array_char		equ		LatSign
sprite_char		equ		Lcaret
bitmap_char		equ		LlBrack
pointer_char	equ		Lasterisk
map_char		equ		LlBrace
data_token		equ		0					;tokens for data areas
dend_token		equ		1
if_token		equ		22					;conditional tokens
then_token		equ		23
ifnd_token		equ		24
fnct_token		equ		18					;function tokens
frun_token		equ		19
bitm_token		equ		34
gkey_token		equ		14
index_start_token	equ	242					;token put at beginning of array/map index expression
index_end_token	equ		243					;token put at end of array/map index expression
lit_str_token	equ		244					;tokens used to identify arguments
lit_num_token	equ		245
keycode_token	equ		246
map_name_token	equ		247
spr_name_token	equ		248
bmp_name_token	equ		249
ptr_name_token	equ		250
str_name_token	equ		251
array_name_token	equ	252
int_name_token	equ		253
eol_token		equ		254					;token put at end of line
eof_token		equ		255					;token put at end of file
dyn_cmd_num		equ		38					;number of commands which have non-fixed handlers
error_str_pos	equ		$0000				;cursor position of error message
error_str_pos_2	equ		$0600
error_str_pos_3	equ		$1200
option_menu_pos	equ		$0212				;cursor position of in-game option menu
option_val_pos	equ		$3036
option_help_pos	equ		$371e
key_delay		equ		12000				;length of delay between keypresses in INPT command
screen_len		equ		768					;length of screen in bytes
full_screen_len	equ		1024
data_seg_len	equ		52					;length of each var segment in data string
array_write_len	equ		25					;max length of array that can be written to file
;trig_precision	equ		15					;precision of trig table in degrees
shft_y_max		equ		20					;maximum distance for SHFT in both directions
shft_x_max		equ		48

;MEMORY ALLOCATION

;permanent memory
copy_addr		equ		$8060				;address where the prog to be executed is copied
end_copy_addr	equ		$ac10
write_addr		equ		$8030				;address where prog is copied to be written back
handler_end_addr	equ	$fba0	;$fa70				;limit for loading command handlers
scrn1_buf		equ		$ca00				;gfx buffers which are actually drawn to screen
scrn2_buf		equ		$fc00
free4_mem		equ		$b000		;560	areas for storing vars, pointers, and sprites
end_free4_mem	equ		$b000+560
free3_mem		equ		$c415		;318
end_free3_mem	equ		$c415+318
free2_mem		equ		$ceef		;180
end_free2_mem	equ		$ceef+180
free1_mem		equ		$c920		;180
end_free1_mem	equ		$c920+180
bmp_mem			equ		$ac10		;1000	area used to store bitmap data
end_bmp_mem		equ		$ac10+1000
spr_mem			equ		$cfc9		;181
func_stack		equ		$c013		;100	stack used to call functions
end_func_stack	equ		$c013+100			;end of function stack
curr_name		equ		$c386		;10		name of current prog's data string
;execution only
vector_table_addr	equ	$c200		;257	address of vector table for isr
isr_addr		equ		$c8c8				;address of grayscale isr
gray2_buf		equ		$bc00		;768	;gfx buffers
gray1_buf		equ		$b900		;768
scr_save2		equ		$b600		;768	buffers where screen is saved for STOP and RCLP
scr_save1		equ		$b300		;768
shft_buf		equ		$b230		;240	(used during y screen scrolling)
wrte_temp		equ		$b230		;		(used while writing to file)
str_temp_ptr	equ		$b230		;2		(used while parsing string expressions)
str_temp_mem	equ		$b232		;35
t_save			equ		$b230		;2		(used while calculating vectors)
ts_save			equ		$b232		;2
;tokenizing only
var_table		equ		$fc00		;991	table used to keep track of vars used in prog
lbl_table		equ		$ca00		;1000	table used to keep track of labels used in prog
lbl_res_ptr		equ		$bbe0		;2		(used when resolving labels)
lbl_res_mem		equ		$bbe2		;750
spr_temp_mem	equ		$bed0		;24		(used when parsing sprite data)
temp_expr_buf	equ		$bed0		;30		area for parsing array args during tokenizing
;temporary memory
shell_copy_addr	equ		$ca00		;731	area where GUI string is copied
stext_copy_addr	equ		$9000		;957	area where shell's text is copied
tokenize_copy_addr	equ	$b000		;3037	area where tokenizer code is copied
err_copy_addr	equ		$b230		;1333	area where error handler code is copied
err_line_copy_addr	equ	$c920		;180	area where text of line with error is copied
;file load (pre-decompress) memory
stext_load_addr	equ		$8100
tokenize_load_addr	equ	$8100
err_load_addr	equ		$ca00
;system data
vars			equ		$c326				;start of storage area for sys vars
temp_buf		equ		vars		;7		temp string buffer
parse_ptr		equ		vars+7		;2		pointer to command currently being parsed
write_ptr		equ		vars+9		;2		pointer to current location in token buffer
curr_var		equ		vars+11		;2		index of current var
curr_var_type	equ		vars+13		;1		type token of current var
curr_var_index	equ		vars+14		;2		index of current var, if array
expr_sum		equ		vars+16		;2		temp accumulator used for sum of expressions
sp_save			equ		vars+18		;2		sp saved here for error
exec_sp			equ		vars+20		;2		sp saved here for if command in exec section needs to be aborted
input_str_sp	equ		vars+22		;2		sp saved here for input string routine
handler_write_ptr	equ	vars+22		;2		pointer to location to write next command handler
func_sp			equ		vars+24		;2		stack pointer for function stack
start_addr		equ		vars+26		;2		pointer to start of function MAIN
bmp_mem_ptr		equ		vars+28		;2		pointer to start of free memory for bitmaps
free1_mem_ptr	equ		vars+30		;2		pointers to start of free memory in var/ptr/sprite storage areas
free2_mem_ptr	equ		vars+32		;2
free3_mem_ptr	equ		vars+34		;2
free4_mem_ptr	equ		vars+36		;2
spr_mem_ptr		equ		vars+38		;2
token_list_ptr	equ		vars+40		;2		pointer to current position in token list
wait_adjust		equ		vars+42		;1		adjustment for WAIT delay
prog_name_save	equ		vars+43		;9		prog name saved here for errors


.org _asm_exec_ram

	nop
	jp prog_start

	.dw 0,shell_title

shell_title: .db "MLC interpreter",0


prog_start:
	set 1,(iy+$16)						;(show table needs to be recalculated,
	res 6,(iy+$1d)						;and that stats are invalid, since we're going to be using memory from both those areas)
;first check for the string with the command handlers (even though it isn't used right now)
	ld hl,exec_str_name-1
	rst 20h
	rst 10h
	ret c
prog_loop:
;then the GUI code and data is copied
	ld hl,shell_str_name-1
	ld b,0
	ld de,shell_copy_addr
	call load_file
	ret c
	ld hl,stext_str_name-1
	ld b,$01
	ld de,stext_load_addr-$8000
	call load_file
	ret c
	ld hl,stext_load_addr
	ld de,stext_copy_addr
	call Decompress
;get the name of the prog from the user
	call shell_copy_addr				;get the name of the test prog to be interpreted,
	di
	push af
	ld de,handler_start_addr			;init some important stuff,
	ld (handler_write_ptr),de
	ld de,token_list
	ld (token_list_ptr),de
	xor a
	ld b,dyn_cmd_num
clear_token_list_loop:
	ld (de),a
	inc de
	djnz clear_token_list_loop
	pop af
	ld (sp_save),sp						;save the stack pointer for any errors that might occur
	jp c,tok_prog
;find the address of its string, or create it if it doesn't exist,
	push hl
	call get_data_str_name
	ld de,curr_name						;save varname
	call _mov10b
	rst 10h								;does it exist?
	jr nc,skip_create_str				;if not, create it
	ld hl,0								;start out at 0 for size
	call _createstrng
skip_create_str:
	pop hl
	push hl
	inc hl								;save the prog's name in case it throws an error
	ld de,prog_name_save
	ld bc,9
	ldir
	pop hl
	call tok_part_1						;run tokenizer in two stages
	ld hl,copy_addr
	ld a,(hl)
	dec a
	call z,exec_tok_prog
	call tok_part_2
;now, finally, execute it!!!
exec_prog:
	call clr_free_mem					;clear the var memory,
	ld hl,str_ti86						;init the model string,
	ld de,free1_mem
	ld bc,str_ti86_len
	ldir
	ld hl,func_stack					;init the function stack,
	ld (func_sp),hl
	xor a
	ld ($c012),a
	call seed_random					;seed the random number generator,
	call clr_buf
	call clr_scrn						;clear the screen,
	call open_gray						;start the grayscale handler,
	call wait_for_no_key				;and wait until no keys are being pressed which could "overflow" into the prog's key handler
	call exec							;then execute!
end_prog:
	call wait_for_no_key
	call close_gray
	call clr_free_mem
	jp prog_loop						;when done with the program, go back to the shell


tok_part_1:
	push hl
	ld hl,tokenize_str_name-1			;then copy the tokenizer code...
	ld b,1
	ld de,tokenize_load_addr-$8000
	call load_file
	jr c,end_tok
	ld hl,tokenize_load_addr
	ld de,tokenize_copy_addr
	call Decompress
	pop hl
;load prog to temp area,
	push hl								;(before loading it, make sure it isn't too big)
	rst 20h
	rst 10h
	call _data_size_to_de
	ld hl,end_copy_addr-copy_addr
	call cphlde
	pop hl
	jr c,end_tok
	ld b,$01
	ld de,copy_addr-$8000
	call load_file						;load the prog,
	jp c,err_noprog
	ld de,copy_addr						;and put 0 at end of file in buffer
	add hl,de
	ld (hl),0
	dec hl								;make sure that there's a newline char at the end of the file
	ld a,(hl)
	cp $d6
	jp nz,insert_newline				;BAD BAD BAD!!!! the user DIDN'T put a newline at the end!!!! we'll just have to do it instead...
	ret
end_tok:
	pop hl
	ret
tok_part_2:
	ld hl,free1_mem						;init all the storage areas,
	ld (free1_mem_ptr),hl
	ld hl,free2_mem
	ld (free2_mem_ptr),hl
	ld hl,free3_mem
	ld (free3_mem_ptr),hl
	ld hl,free4_mem
	ld (free4_mem_ptr),hl
	ld hl,spr_1_data
	ld (spr_mem_ptr),hl
	ld hl,bmp_mem
	ld (bmp_mem_ptr),hl
	ld hl,lbl_res_mem
	ld (lbl_res_ptr),hl
	ld a,3
	ld (wait_adjust),a
;	ld a,$01
;	out (3),a
	call tokenize_copy_addr				;tokenize and resolve labels
;	ld a,$0b
;	out (3),a
	ret


insert_newline:
	inc hl
	ld (hl),$d6
	inc hl
	ld (hl),0
	ret

get_data_str_name:
	rst 20h								;op1=varname
	ld hl,_op1+8
	ld de,_op1+9
	ld bc,7
	lddr
	ld a,Ltheta
	ld (de),a
	ld a,(hl)
	cp 8
	jp z,err_progname
	inc (hl)
	dec hl								;op1=[theta]varname
	ret

clr_free_mem:
	ld hl,free1_mem
	ld bc,end_free1_mem-free1_mem-1
	call clr_free_mem_hl
	ld hl,free2_mem
	ld bc,end_free2_mem-free2_mem-1
	call clr_free_mem_hl
	ld hl,free3_mem
	ld bc,end_free3_mem-free3_mem-1
	call clr_free_mem_hl
	ld hl,free4_mem
	ld bc,end_free4_mem-free4_mem-1
	call clr_free_mem_hl
	ld hl,vector_table_addr				;not needed for program, prevents crash on exit
	ld bc,257
	call clr_free_mem_hl
	ld hl,spr_mem+1
	ld bc,179
clr_free_mem_hl:
	ld d,h
	ld e,l
	inc de
	ld (hl),0
	ldir
	ret

;Other code sections

#include "exec.asm"						;token executor
#include "pretok.asm"					;pre-tokenized program tokenize/exec
#include "var.asm"						;variable management
#include "sprite.asm"					;sprite management
#include "ptr.asm"						;pointer management
#include "err.asm"						;errors
#include "FVPUTS.inc"					;Josh Seagoe's text routine
#include "gfx.asm"						;graphics routines
#include "routines.asm"					;other useful routines
#include "data.asm"						;uhh...data!
#include "int.asm"						;interrupt-related code

handler_start_addr:

.end